#ifndef PETSPECTFUSIONAPP_H
#define PETSPECTFUSIONAPP_H

/**
 *	$Id$
 *  $Lastlog$
 *
 *\brief This is the main application namespace containing global definitions and functions
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@in.tum.de
 *
 * \ingroup PetSpectFusion
 * \version 1.30
 * \date 17.11.2009
 *
 * \par License:
 * Copyright (c) 2007 - 2009,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 */

#include "typedefs.h"

class QPetSpectFusionViewer;

namespace PetSpectFusionApp {

	extern float sampleRate;
	extern float minStepSize;
	extern float maxStepSize;
	
	extern int bins;
	extern int maxIterations;
	extern int levels;
	extern int zoomFactor;
	
	extern std::string fixedFileName;
	extern std::string movingFileName;
	extern std::string outputFileName;
	extern std::string plotFileName;
	extern std::string logFileName;

	extern bool doMultiRes;
	extern bool automatic_mode;
	extern bool transform_mode;
	extern bool graphics_mode;
	
	extern ParametersType transformParams;
	extern OptimizerScalesType optimizerScales;
	extern TransformType::Pointer transform;

	//used for taking the window parameters from a selected region of the images
	extern RegionType fixedWindowRegion;
	
	//used for VOI registration
	extern RegionType voiRegion;
	extern bool doVOIRegistration;
	extern bool doVOIAndFullRegistration;
	
	extern QPetSpectFusionViewer *mainViewer;

	void init();
	int processCommandLine(int argc, char** argv);
	
	RegionType processWindowRegion(char** argv);
	
	RegionType processVOIRegion(char** argv);
	
}


#endif //PETSPECTFUSIONAPP_H