#include <QtGui>

#include "typedefs.h"
#include "RegistrationView.h"
#include "ViewObserver.h"

/**
 *	$Id$
 *  $Lastlog$
 *
 * \brief This class contains the advanced registration settings dialog
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 * \ingroup PetSpectFusion
 * \version 1.0
 * \date 02.11.2009
 *
 * \par License:
 * Copyright (c) 2008 - 2009,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 *
 */
class RegistrationViewDialog : public QDialog, public  ViewObserver
{
	Q_OBJECT
	
	public:
		RegistrationViewDialog(QWidget *parent = 0, QRegistrationView* registrationView = 0);
	
		QPushButton *applyButton;
		QPushButton *revertButton;
	
		QDoubleSpinBox* fixedMaxPerBox;
		QDoubleSpinBox* fixedMinPerBox;
		QDoubleSpinBox* fixedMaxValBox;
		QDoubleSpinBox* fixedMinValBox;
		QSpinBox* fixedPrecisionBox;
		QComboBox* fixedColorCombo;
		QRadioButton* fixedModeMaxButton;
		QRadioButton* fixedModeMedianButton;
	
		QDoubleSpinBox* movingMaxPerBox;
		QDoubleSpinBox* movingMinPerBox;
		QDoubleSpinBox* movingMaxValBox;
		QDoubleSpinBox* movingMinValBox;
		QSpinBox* movingPrecisionBox;
		QComboBox* movingColorCombo;
		QDoubleSpinBox* movingModeMaxButton;
		QRadioButton* movingModeMedianButton;

		QRadioButton* manualWindowButton;
		QRadioButton* regionWindowButton;
		QSpinBox* indexX;
		QSpinBox* indexY;
		QSpinBox* indexZ;
		QSpinBox* rangeX;
		QSpinBox* rangeY;
		QSpinBox* rangeZ;
		QCheckBox* showRegionBox;
	
		//ViewObserver methods
		void fixedImageChanged();
		void movingImageChanged();
	
	private:
		QRegistrationView* regView;
	
	private slots:
		void manualWindowMode();
		void regionWindowMode();
		void fixedMaxPerBoxChanged(double val);
		void fixedMaxValBoxChanged(double val);
		void fixedMinPerBoxChanged(double val);
		void fixedMinValBoxChanged(double val);
		void fixedPrecisionChanged(int val);
	
		void movingMaxPerBoxChanged(double val);
		void movingMaxValBoxChanged(double val);
		void movingMinPerBoxChanged(double val);
		void movingMinValBoxChanged(double val);
		void movingPrecisionChanged(int val);
	
		void apply();
		void revert();
	
};
