//
//  $Id$
//  $Lastlog$
//
//  CheckerboardDialog.cxx
//  PetSpectFusion
//
//	Created 26/11/2009, Implements the checkerboard view dialog.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "Log.h"
#include "CheckerboardViewDialog.h"


CheckerboardViewDialog::CheckerboardViewDialog(QWidget *parent, QRegistrationView* registrationView) : QDialog(parent)
{
	regView = registrationView;
	
	QVBoxLayout *mainBox = new QVBoxLayout;

	
	QGroupBox* checkerBox = new QGroupBox(QString("Checkerboard Composite Options"));
	QGridLayout* checkerLayout = new QGridLayout;
	QVBoxLayout* checkerOrderLayout = new QVBoxLayout;
	
	CheckerBoardType::PatternArrayType pattern = regView->getCheckerBoardPattern(); 
	
	QLabel* checkerXLabel = new QLabel(tr("Checker sqaures along X"));
	checkerLayout->addWidget(checkerXLabel, 0, 0);
	checkerXBox = new QDoubleSpinBox;
	checkerXBox->setSingleStep(1.0);
	checkerXBox->setDecimals(0);
	checkerXBox->setRange(1, 15);
	checkerXBox->setValue(pattern[0]);
	checkerLayout->addWidget(checkerXBox, 0, 1);
	
	QLabel* checkerYLabel = new QLabel(tr("Checker sqaures along Y"));
	checkerLayout->addWidget(checkerYLabel, 1, 0);
	checkerYBox = new QDoubleSpinBox;
	checkerYBox->setSingleStep(1.0);
	checkerYBox->setDecimals(0);
	checkerYBox->setRange(1, 15);
	checkerYBox->setValue(pattern[1]);
	checkerLayout->addWidget(checkerYBox, 1, 1);
	
	QLabel* checkerZLabel = new QLabel(tr("Checker sqaures along Z"));
	checkerLayout->addWidget(checkerZLabel, 2, 0);
	checkerZBox = new QDoubleSpinBox;
	checkerZBox->setSingleStep(1.0);
	checkerZBox->setDecimals(0);
	checkerZBox->setRange(1, 15);
	checkerZBox->setValue(pattern[2]);
	checkerLayout->addWidget(checkerZBox, 2, 1);
	
	fixedFirstButton = new QRadioButton(tr("Fixed image displayed first"));
	checkerOrderLayout->addWidget(fixedFirstButton);
	
	movingFirstButton = new QRadioButton(tr("Moving image displayed first"));
	checkerOrderLayout->addWidget(movingFirstButton);
	
	fixedFirstButton->setChecked(regView->getCheckerBoardOrder());
	checkerLayout->addLayout(checkerOrderLayout, 3, 0, 1, 2);
	checkerBox->setLayout(checkerLayout);
	mainBox->addWidget(checkerBox);

	QHBoxLayout* buttonsLayout = new QHBoxLayout;
	
	revertButton = new QPushButton(tr("Revert"));
	revertButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonsLayout->addWidget(revertButton);
	connect(revertButton, SIGNAL(clicked()), this, SLOT(revert()));
	
	applyButton = new QPushButton(tr("Apply"));
	applyButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	applyButton->setDefault(true);
	buttonsLayout->addWidget(applyButton);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	
	mainBox->addItem(buttonsLayout);
	setLayout(mainBox);	
	setWindowTitle(tr("Checkerboard View  Options"));	
}

void CheckerboardViewDialog::apply()
{
	logging::logMessage("Apply new checkerboard parameters");
	
	regView->setCheckerBoardOrder(fixedFirstButton->isChecked());
	
	CheckerBoardType::PatternArrayType pattern;
	pattern[0] = checkerXBox->value();
	pattern[1] = checkerYBox->value();
	pattern[2] = checkerZBox->value();
	regView->setCheckerBoardPattern(pattern);
	
	regView->updateView();
}

void CheckerboardViewDialog::revert()
{
	logging::logMessage("Reverting to default checkerboard parameters");
	
	CheckerBoardType::PatternArrayType pattern;
	pattern[0] = DEFAULT_CHECKER_SQUARES;
	pattern[1] = DEFAULT_CHECKER_SQUARES;
	pattern[2] = DEFAULT_CHECKER_SQUARES;
	checkerXBox->setValue(DEFAULT_CHECKER_SQUARES);
	checkerYBox->setValue(DEFAULT_CHECKER_SQUARES);
	checkerZBox->setValue(DEFAULT_CHECKER_SQUARES);
	regView->setCheckerBoardPattern(pattern);
	fixedFirstButton->setChecked(regView->getCheckerBoardOrder());
	
	regView->updateView();
}


