//
// $Id$
// $Lastlog$
//
//  Log.cxx
//  PetSpectFusion
//
//	Created 26/10/2009, These are various log functions
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "Log.h"
#include <iostream>
#include <fstream>
#include <time.h>

using namespace std;

#define TIME_FORMAT "%d/%m/%Y %H:%M:%S"

namespace logging {

static ofstream logFile;
static string logFileName;
static streambuf *outbuf, *errbuf;

void logMessage(string & msg)
{
	time_t rawtime;
	string* timeString;
	char timeBuf[26];
	struct tm* timeinfo;
	
	//Grab the current time
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	
	//format it
	strftime(timeBuf, 25, TIME_FORMAT, timeinfo);
	timeString = new string(timeBuf);
	
	if(timeString != NULL)
	{
		cout << "(" << *timeString << "): " << msg << endl;
		delete timeString;
	}
}

void logMessage(const char* msg)
{
	string cppString(msg);
	logMessage(cppString);
}

void logMessage(ostringstream& msg)
{
	string str = msg.str();
	logMessage(str);
}

void logError(std::string & msg)
{
	time_t rawtime;
	string* timeString;
	char timeBuf[26];
	struct tm* timeinfo;
	
	//Grab the current time
	time(&rawtime);
	timeinfo = localtime(&rawtime);
	
	//format it
	strftime(timeBuf, 25, TIME_FORMAT, timeinfo);
	timeString = new string(timeBuf);
	
	if(timeString != NULL)
	{
		cerr << "(" << *timeString << "): " << msg << endl;
		delete timeString;
	}

}

void logError(const char* msg)
{
	string cppString(msg);
	logError(cppString);
}

void logError(ostringstream& msg)
{
	string cppString = msg.str();
	logError(cppString);
}

void enableFileLogging(string& fileName)
{
	//redirect the standard out and error if logging is wished
	if(fileName.size() > 0)
	{
		logFile.open(fileName.c_str());
		if(logFile.is_open())
		{
			logFileName = fileName;
			outbuf = std::cout.rdbuf();
			errbuf = std::cerr.rdbuf();
			std::cout.rdbuf(logFile.rdbuf());
			std::cerr.rdbuf(logFile.rdbuf());
			
			logMessage("Logging started");
		}
		else 
		{
			std::ostringstream ostring(std::ostringstream::out);
			ostring << "Error opening log file " << fileName << ", logging is disabled!";
			logError(ostring);
		}
	}
	
}

void disableFileLogging()
{
	if(logFile.is_open())
	{
		cout.rdbuf(outbuf);
		cerr.rdbuf(errbuf);
		logFile.close();
	}
}

bool fileLoggingEnabled()
{
	return logFile.is_open();
}
	
string& getLogFileName()
{
	return logFileName;
		
}
	
}
