//
//  $Id$
//  $Lastlog$
//
//  RegSettingsDialog.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements a dialog for changing various settings in PetSpectFusion
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "RegSettingsDialog.h"

RegSettingsDialog::RegSettingsDialog(QPetSpectFusionViewer *parent) : QDialog(parent)
{
	QVBoxLayout *mainBox = new QVBoxLayout;
	
	QGroupBox *settingsBox = new QGroupBox(QString("Optimizer and Metric Parameters"));
	QGridLayout *settingsLayout = new QGridLayout;
	
	QLabel* binsLabel = new QLabel(tr("Bins"));
	settingsLayout->addWidget(binsLabel, 0, 0);
	binsBox = new QSpinBox;
	binsBox->setRange(5, 1000);
	binsBox->setSingleStep(1);
	binsBox->setValue(DEFAULT_BINS);
	settingsLayout->addWidget(binsBox, 0 , 1);
	
	QLabel *samplesLabel = new QLabel(tr("Sample Rate"));
	settingsLayout->addWidget(samplesLabel, 0, 2);
	samplesBox = new QDoubleSpinBox;
	samplesBox->setRange(0.01, 100.0);
	samplesBox->setDecimals(2);
	samplesBox->setValue(DEFAULT_SAMPLERATE);
	samplesBox->setSingleStep(1.0);
	settingsLayout->addWidget(samplesBox, 0, 3);
	
	QLabel *minStepLabel = new QLabel(tr("Min Step Size"));
	settingsLayout->addWidget(minStepLabel, 1, 0);
	minSizeBox = new QDoubleSpinBox;
	minSizeBox->setRange(0.001, 50.0);
	minSizeBox->setDecimals(3);
	minSizeBox->setValue(DEFAULT_MINSTEP);
	settingsLayout->addWidget(minSizeBox, 1, 1);
	
	QLabel* maxStepLabel = new QLabel(tr("Max Step Size"));
	settingsLayout->addWidget(maxStepLabel, 1, 2);
	maxSizeBox = new QDoubleSpinBox;
	maxSizeBox->setRange(0.001, 50.0);
	maxSizeBox->setDecimals(3);
	maxSizeBox->setValue(DEFAULT_MAXSTEP);
	settingsLayout->addWidget(maxSizeBox, 1, 3);
	
	settingsBox->setLayout(settingsLayout);
	mainBox->addWidget(settingsBox);
	
	QGridLayout *loggingGrid = new QGridLayout;
	QLabel* loggingLabel = new QLabel(tr("Enable Logging"));
	loggingGrid->addWidget(loggingLabel, 0, 0);
	loggingBox = new QCheckBox;
	loggingGrid->addWidget(loggingBox, 0, 1);
	logNameBox = new QLineEdit;
	loggingGrid->addWidget(logNameBox, 0, 2);
	loggingButton = new QPushButton(tr("Log file"));
	loggingButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	loggingGrid->addWidget(loggingButton, 0, 3);
	
	QLabel* plotsLabel = new QLabel(tr("Enable Plots"));
	loggingGrid->addWidget(plotsLabel, 1, 0);
	plotsBox = new QCheckBox;
	loggingGrid->addWidget(plotsBox, 1, 1);
	plotsNameBox = new QLineEdit;
	loggingGrid->addWidget(plotsNameBox, 1, 2);
	plotsButton = new QPushButton(tr("Plots file"));
	plotsButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	loggingGrid->addWidget(plotsButton, 1, 3);
	
	QGroupBox *logplotBox = new QGroupBox(QString("Logging / Plotting"));
	logplotBox->setLayout(loggingGrid);
	mainBox->addWidget(logplotBox);
	
	//optimizer scales grid
	QGridLayout *scalesLayout = new QGridLayout;
	QLabel *rotLabel = new QLabel(tr("Rotation"));
	scalesLayout->addWidget(rotLabel, 0, 1);
	scalesLayout->setColumnStretch(1, 1);
	QLabel *translLabel = new QLabel(tr("Translation"));
	scalesLayout->addWidget(translLabel, 0, 2);
	scalesLayout->setColumnStretch(2, 1);

	QLabel *xLabel = new QLabel(tr("X"));
	scalesLayout->addWidget(xLabel, 1, 0);
	xRotBox = new QDoubleSpinBox;
	xRotBox->setRange(0.00000001, 10.0);
	xRotBox->setDecimals(7);
	scalesLayout->addWidget(xRotBox, 1, 1);
	xTransBox = new QDoubleSpinBox;
	xTransBox->setRange(0.0000001, 10.0);
	xTransBox->setDecimals(7);
	scalesLayout->addWidget(xTransBox, 1, 2);
	
	QLabel *yLabel = new QLabel(tr("Y"));
	scalesLayout->addWidget(yLabel, 2, 0);
	yRotBox = new QDoubleSpinBox;
	yRotBox->setRange(0.0000001, 10.0);
	yRotBox->setDecimals(7);
	scalesLayout->addWidget(yRotBox, 2, 1);
	yTransBox = new QDoubleSpinBox;
	yTransBox->setRange(0.0000001, 10.0);
	yTransBox->setDecimals(7);
	scalesLayout->addWidget(yTransBox, 2, 2);
	
	QLabel *zLabel = new QLabel(tr("Z"));
	scalesLayout->addWidget(zLabel, 3, 0);
	zRotBox = new QDoubleSpinBox;
	zRotBox->setRange(0.0000001, 10.0);
	zRotBox->setDecimals(7);
	scalesLayout->addWidget(zRotBox, 3, 1);
	zTransBox = new QDoubleSpinBox;
	zTransBox->setRange(0.0000001, 10.0);
	zTransBox->setDecimals(7);
	scalesLayout->addWidget(zTransBox, 3, 2);
	
	QGroupBox *scalesBox = new QGroupBox(QString("Optimizer Scales"));
	scalesBox->setLayout(scalesLayout);
	mainBox->addWidget(scalesBox);
	
	QGroupBox* voiBox = new QGroupBox(QString("Registration Mode"));
	QGridLayout* voiLayout = new QGridLayout;
	
	QLabel* indexLabel = new QLabel(tr("Index"));
	voiLayout->addWidget(indexLabel, 0, 1, Qt::AlignHCenter);
	voiLayout->setColumnStretch(1, 1);
	QLabel* rangeLabel = new QLabel(tr("Range"));
	voiLayout->addWidget(rangeLabel, 0, 2, Qt::AlignHCenter);
	voiLayout->setColumnStretch(2, 1);
		
	xLabel = new QLabel(tr("X"));
	voiLayout->addWidget(xLabel, 1, 0);
	
	indexX = new QSpinBox;
	indexX->setRange(-1000, 1000);
	indexX->setSingleStep(1);
	indexX->setValue(0);
	voiLayout->addWidget(indexX, 1, 1);
	
	rangeX = new QSpinBox;
	rangeX->setSingleStep(1);
	rangeX->setRange(-1000, 1000);
	rangeX->setValue(0);
	voiLayout->addWidget(rangeX, 1, 2);
	
	yLabel = new QLabel(tr("Y"));
	voiLayout->addWidget(yLabel, 2, 0);
	
	indexY = new QSpinBox;
	indexY->setSingleStep(1);
	indexY->setRange(-1000, 1000);
	indexY->setValue(0);
	voiLayout->addWidget(indexY, 2, 1);
	
	rangeY = new QSpinBox;
	rangeY->setSingleStep(1);
	rangeY->setRange(-1000, 1000);
	rangeY->setValue(0);
	voiLayout->addWidget(rangeY, 2, 2);
	
	zLabel = new QLabel(tr("Z"));
	voiLayout->addWidget(zLabel, 3, 0);
	
	indexZ = new QSpinBox;
	indexZ->setSingleStep(1);
	indexZ->setRange(-1000, 1000);
	indexZ->setValue(0);
	voiLayout->addWidget(indexZ, 3, 1);
	
	rangeZ = new QSpinBox;
	rangeZ->setSingleStep(1);
	rangeZ->setRange(-1000, 1000);
	rangeZ->setValue(0);
	voiLayout->addWidget(rangeZ, 3, 2);
		
	voiBox->setLayout(voiLayout);
	mainBox->addWidget(voiBox);
	
	//buttons
	QGridLayout *buttonsGrid = new QGridLayout;
	applyButton = new QPushButton(tr("Apply"));
	applyButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	applyButton->setDefault(true);
	buttonsGrid->addWidget(applyButton, 0, 0);
	connect(applyButton, SIGNAL(clicked()), parent, SLOT(applyNewSettings()));
	
	revertButton = new QPushButton(tr("Revert"));
	revertButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonsGrid->addWidget(revertButton, 0, 1);
	connect(revertButton, SIGNAL(clicked()), this, SLOT(revert()));
	
	connect(loggingButton, SIGNAL(clicked()), this, SLOT(chooseLogFile()));
	connect(plotsButton, SIGNAL(clicked()), this, SLOT(choosePlotsFile()));
	mainBox->addLayout(buttonsGrid);
	
	setLayout(mainBox);
	setWindowTitle(tr("Options"));

}

void RegSettingsDialog::chooseLogFile()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Log File"),
                            "", tr("Text files (*.txt)"));
	logNameBox->setText(fileName);
}

void RegSettingsDialog::choosePlotsFile()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Plot file base"),
                            "", tr("Just a prefix, endings added automatically"));
	plotsNameBox->setText(fileName);
}

void RegSettingsDialog::revert()
{
	//TODO!
}

