//
//  $Id$
//  $Lastlog$
//
//  RegistrationThread.cxx
//  PetSpectFusion
//
//	Created 12/02/2008, Implements various functions for reading and writing images
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "Registration.h"
#include "RegObserver.h"

//needed so that Qt can send the ParametersType in a signal
Q_DECLARE_METATYPE(ParametersType); 

QRegistrationThread::QRegistrationThread(QObject *parent, QPetSpectFusionViewer* viewer, InternalImageType::Pointer fImage, InternalImageType::Pointer mImage, 
		int numBins, double minStep, double maxStep, float samples, int maxIter, OptimizerScalesType scales, ParametersType tParams)
	: QThread(parent)
{
	fixedImage = fImage;
	movingImage = mImage;
	display = viewer;
	params = tParams;
	bins = numBins;
	minSize = minStep;
	maxSize = maxStep;
	sampleRate = samples;
	maxIterations = maxIter;
	optimizerScales = scales;
	
	plotName = "";

	multiRes = false;
	
	//registering the type, so we can send it with a signal
	qRegisterMetaType<ParametersType>("ParametersType");

}

void QRegistrationThread::run()
{	
	RegistrationResultType result;
	
	observer = CommandIterationUpdate::New();
	observer->setDisplayThread(this);
	
	if(plotName != "")
	{
		observer->enablePlotting(plotName.c_str());
	}
	
	if(multiRes)
	{
		result = doMattesMultiRegistration(fixedImage, movingImage, observer, bins,
				sampleRate , minSize, maxSize, maxIterations, optimizerScales, levels, params);
	}
	else
	{
		result = doMattesRegistration(fixedImage, movingImage, observer, bins,
				sampleRate , minSize, maxSize, maxIterations, optimizerScales, params);
		
	}		
	
	if(plotName != "")
		observer->closePlotFiles();
	
	if (!result.error) {
		emit parametersUpdated(result.params, result.metricVal, result.iteration);
	}
}

/**
 *	function emits a signal for the main GUI thread
 *
 */
void QRegistrationThread::registrationUpdate(ParametersType params, double metricVal, int iteration)
{
	emit parametersUpdated(params, metricVal, iteration);
}

/**
 *	Called by the GUI to stop registration. Function effectively tells the optimizer to stop.
 *
 */
void QRegistrationThread::stopRegistration()
{
	observer->stopRegistration();
}

void QRegistrationThread::enablePlotting(std::string fileName)
{
	plotName = fileName;
}

void QRegistrationThread::setMultiRes(bool set, int lev)
{
	levels = lev;
	multiRes = set;
}
