//
//  $Id$
//  $Lastlog$
//
//  RegistrationViewDialog.cxx
//  PetSpectFusion
//
//	Created 02/11/2009, Implements various functions for reading and writing images
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <math.h>

#include "Log.h"
#include "RegistrationViewDialog.h"

RegistrationViewDialog::RegistrationViewDialog(QWidget *parent, QRegistrationView* registrationView) : QDialog(parent)
{
	regView = registrationView;
	
	QVBoxLayout *mainBox = new QVBoxLayout;
	
	QGroupBox* modeBox = new QGroupBox(QString("Windowing Mode"));
	QVBoxLayout* modeLayout = new QVBoxLayout;
	
	manualWindowButton = new QRadioButton(tr("Manually set window parameters"));
	modeLayout->addWidget(manualWindowButton);
	regionWindowButton = new QRadioButton(tr("Set window from fixed image region"));
	modeLayout->addWidget(regionWindowButton);
	modeBox->setLayout(modeLayout);	
	mainBox->addWidget(modeBox);
	
	QGroupBox* fixedWinGroupBox = new QGroupBox(QString("Fixed Image Options"));
	QGridLayout* fixedWinLayout = new QGridLayout;
	fixedWinLayout->setSpacing(5);
	
	QLabel* fixedMaxLabel = new QLabel(tr("Window max percent"));
	fixedWinLayout->addWidget(fixedMaxLabel, 0, 0);
	fixedMaxPerBox = new QDoubleSpinBox;
	fixedMaxPerBox->setSingleStep(1.0);
	fixedMaxPerBox->setDecimals(2);
	fixedMaxPerBox->setRange(-10000, 10000);
	fixedMaxPerBox->setValue(regView->getFixedWindowMax()*100.0);
	fixedWinLayout->addWidget(fixedMaxPerBox, 0, 1);
	
	connect(fixedMaxPerBox, SIGNAL(valueChanged(double)), this, SLOT(fixedMaxPerBoxChanged(double)));
	
	QLabel* fixedMaxValLabel = new QLabel(tr("Window max value"));
	fixedWinLayout->addWidget(fixedMaxValLabel, 1, 0);
	fixedMaxValBox = new QDoubleSpinBox;
	fixedWinLayout->addWidget(fixedMaxValBox, 1, 1);
	
	connect(fixedMaxValBox, SIGNAL(valueChanged(double)), this, SLOT(fixedMaxValBoxChanged(double)));
	
	QLabel* fixedMinLabel = new QLabel(tr("Window min percent"));
	fixedWinLayout->addWidget(fixedMinLabel, 2, 0);
	fixedMinPerBox = new QDoubleSpinBox;
	fixedMinPerBox->setSingleStep(1.0);
	fixedMinPerBox->setDecimals(2);
	fixedMinPerBox->setRange(-10000, 10000);
	fixedMinPerBox->setValue(regView->getFixedWindowMin()*100.0);
	fixedWinLayout->addWidget(fixedMinPerBox, 2, 1);
	
	connect(fixedMinPerBox, SIGNAL(valueChanged(double)), this, SLOT(fixedMinPerBoxChanged(double)));
	
	QLabel* fixedMinValLabel = new QLabel(tr("Window min value"));
	fixedWinLayout->addWidget(fixedMinValLabel, 3, 0);
	fixedMinValBox = new QDoubleSpinBox;
	fixedWinLayout->addWidget(fixedMinValBox, 3, 1);
	
	connect(fixedMinValBox, SIGNAL(valueChanged(double)), this, SLOT(fixedMinValBoxChanged(double)));

	QLabel *fixedPrecisionLabel = new QLabel(tr("Window precision"));
	fixedWinLayout->addWidget(fixedPrecisionLabel, 4, 0);
	fixedPrecisionBox = new QSpinBox;
	fixedPrecisionBox->setRange(1, 15);
	fixedPrecisionBox->setValue(6);
	fixedWinLayout->addWidget(fixedPrecisionBox, 4, 1);
	
	connect(fixedPrecisionBox, SIGNAL(valueChanged(int)), this, SLOT(fixedPrecisionChanged(int)));
	
	QLabel* fixedColorLabel = new QLabel(tr("Color Table"));
	fixedWinLayout->addWidget(fixedColorLabel, 5, 0);
	fixedColorCombo = new QComboBox;
	
	for (int i = 0; i < ColorTableCount; i++)
		fixedColorCombo->addItem(tr(ColorTableNames[i]));
	
	fixedColorCombo->setCurrentIndex(regView->getFixedImageColorTable());
	fixedWinLayout->addWidget(fixedColorCombo, 5, 1);
	
	fixedWinGroupBox->setLayout(fixedWinLayout);
	mainBox->addWidget(fixedWinGroupBox);
	
	QGroupBox* movingWinGroupBox = new QGroupBox(QString("Moving Image Options"));
	QGridLayout* movingWinLayout = new QGridLayout;
	movingWinLayout->setSpacing(5);
	
	QLabel* movingMaxLabel = new QLabel(tr("Window max percent"));
	movingWinLayout->addWidget(movingMaxLabel, 0, 0);
	movingMaxPerBox = new QDoubleSpinBox;
	movingMaxPerBox->setSingleStep(1.0);
	movingMaxPerBox->setDecimals(2);
	movingMaxPerBox->setRange(-10000, 10000);
	movingMaxPerBox->setValue(regView->getMovingWindowMax()*100.0);
	movingWinLayout->addWidget(movingMaxPerBox, 0, 1);
	
	connect(movingMaxPerBox, SIGNAL(valueChanged(double)), this, SLOT(movingMaxPerBoxChanged(double)));
	
	QLabel* movingMaxValLabel = new QLabel(tr("Window max value"));
	movingWinLayout->addWidget(movingMaxValLabel, 1, 0);
	movingMaxValBox = new QDoubleSpinBox;
	movingWinLayout->addWidget(movingMaxValBox, 1, 1);	
	
	connect(movingMaxValBox, SIGNAL(valueChanged(double)), this, SLOT(movingMaxValBoxChanged(double)));
	
	QLabel* movingMinLabel = new QLabel(tr("Window min percent"));
	movingWinLayout->addWidget(movingMinLabel, 2, 0);
	movingMinPerBox = new QDoubleSpinBox;
	movingMinPerBox->setSingleStep(1.0);
	movingMinPerBox->setDecimals(2);
	movingMinPerBox->setRange(-10000, 10000);
	movingMinPerBox->setValue(regView->getMovingWindowMin()*100.0);
	movingWinLayout->addWidget(movingMinPerBox, 2, 1);
	
	connect(movingMinPerBox, SIGNAL(valueChanged(double)), this, SLOT(movingMinPerBoxChanged(double)));
	
	QLabel* movingMinValLabel = new QLabel(tr("Window min value"));
	movingWinLayout->addWidget(movingMinValLabel, 3, 0);
	movingMinValBox = new QDoubleSpinBox;
	movingWinLayout->addWidget(movingMinValBox, 3, 1);
	
	connect(movingMinValBox, SIGNAL(valueChanged(double)), this, SLOT(movingMinValBoxChanged(double)));
	
	QLabel* movingPrecisionLabel = new QLabel(tr("Window precision"));
	movingWinLayout->addWidget(movingPrecisionLabel, 4, 0);
	movingPrecisionBox = new QSpinBox;
	movingPrecisionBox->setRange(0, 15);
	movingPrecisionBox->setValue(6);
	movingWinLayout->addWidget(movingPrecisionBox, 4, 1);
	
	connect(movingPrecisionBox, SIGNAL(valueChanged(int)), this, SLOT(movingPrecisionChanged(int)));
	
	QLabel* movingColorLabel = new QLabel(tr("Color Table"));
	movingWinLayout->addWidget(movingColorLabel, 5, 0);
	movingColorCombo = new QComboBox;
	
	for (int i = 0; i < ColorTableCount; i++)
		movingColorCombo->addItem(tr(ColorTableNames[i]));
	
	movingColorCombo->setCurrentIndex(regView->getMovingImageColorTable());
	movingWinLayout->addWidget(movingColorCombo, 5, 1);
	
	movingWinGroupBox->setLayout(movingWinLayout);
	mainBox->addWidget(movingWinGroupBox);	
	
	QGroupBox* regionBox = new QGroupBox(QString("Fixed Image Window Region Settings"));
	QGridLayout* regionLayout = new QGridLayout;
	regionLayout->setSpacing(5);
	
	QLabel* indexLabel = new QLabel(tr("Index"));
	regionLayout->addWidget(indexLabel, 0, 1, Qt::AlignHCenter);
	regionLayout->setColumnStretch(1, 1);
	QLabel* rangeLabel = new QLabel(tr("Range"));
	regionLayout->addWidget(rangeLabel, 0, 2, Qt::AlignHCenter);
	regionLayout->setColumnStretch(2, 1);
	
	RegionType region = regView->getFixedWindowRegion();
	
	QLabel* xLabel = new QLabel(tr("X"));
	regionLayout->addWidget(xLabel, 1, 0);
	
	indexX = new QSpinBox;
	indexX->setRange(0, 1000);
	indexX->setSingleStep(1);
	indexX->setValue(region.GetIndex(0));
	regionLayout->addWidget(indexX, 1, 1);
	
	rangeX = new QSpinBox;
	rangeX->setSingleStep(1);
	rangeX->setRange(-1000, 1000);
	rangeX->setValue(region.GetSize(1));
	regionLayout->addWidget(rangeX, 1, 2);
	
	QLabel* yLabel = new QLabel(tr("Y"));
	regionLayout->addWidget(yLabel, 2, 0);
	
	indexY = new QSpinBox;
	indexY->setSingleStep(1);
	indexY->setRange(-1000, 1000);
	indexY->setValue(region.GetIndex(2));
	regionLayout->addWidget(indexY, 2, 1);
	
	rangeY = new QSpinBox;
	rangeY->setSingleStep(1);
	rangeY->setRange(-1000, 1000);
	rangeY->setValue(region.GetSize(1));
	regionLayout->addWidget(rangeY, 2, 2);
	
	QLabel* zLabel = new QLabel(tr("Z"));
	regionLayout->addWidget(zLabel, 3, 0);
	
	indexZ = new QSpinBox;
	indexZ->setSingleStep(1);
	indexZ->setRange(-1000, 1000);
	indexZ->setValue(region.GetIndex(2));
	regionLayout->addWidget(indexZ, 3, 1);
	
	rangeZ = new QSpinBox;
	rangeZ->setSingleStep(1);
	rangeZ->setRange(-1000, 1000);
	rangeZ->setValue(region.GetSize(2));
	regionLayout->addWidget(rangeZ, 3, 2);
	
	showRegionBox = new QCheckBox(tr("Show window region"));
	showRegionBox->setChecked(false);
	regionLayout->addWidget(showRegionBox, 4, 0, 1,	3);
	
	regionBox->setLayout(regionLayout);
	mainBox->addWidget(regionBox);
	connect(manualWindowButton, SIGNAL(clicked()), this, SLOT(manualWindowMode()));
	connect(regionWindowButton, SIGNAL(clicked()), this, SLOT(regionWindowMode()));	
	
	QHBoxLayout* buttonsLayout = new QHBoxLayout;
	
	revertButton = new QPushButton(tr("Revert"));
	revertButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonsLayout->addWidget(revertButton);
	connect(revertButton, SIGNAL(clicked()), this, SLOT(revert()));
	
	applyButton = new QPushButton(tr("Apply"));
	applyButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	applyButton->setDefault(true);
	buttonsLayout->addWidget(applyButton);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	
	mainBox->addItem(buttonsLayout);
	setLayout(mainBox);	
	setWindowTitle(tr("Advanced View Options"));
	
	if(parent)
		connect(this, SIGNAL(destroyed()), parent, SLOT(regViewDialogClosed()));
	
	fixedImageChanged();
	movingImageChanged();
	
	if(regView->getWindowingMode())
	{
		regionWindowButton->setChecked(true);
		regionWindowMode();
	}
	else
	{
		manualWindowButton->setChecked(true);
		manualWindowMode();
	}
	
	regView->addObserver(this);
}

void RegistrationViewDialog::apply()
{
	logging::logMessage("Applying new intensity window values");
	
	if(manualWindowButton->isChecked())
	{
		regView->setFixedWindowMax(fixedMaxPerBox->value()/100.0);
		regView->setFixedWindowMin(fixedMinPerBox->value()/100.0);
		regView->setMovingWindowMax(movingMaxPerBox->value()/100.0);
		regView->setMovingWindowMin(movingMinPerBox->value()/100.0);
		
		regView->setWindowingMode(false);
	}
	else 
	{
		RegionType region;
		region.SetIndex(0, indexX->value());
		region.SetIndex(1, indexY->value());
		region.SetIndex(2, indexZ->value());
		region.SetSize(0, rangeX->value());
		region.SetSize(1, rangeY->value());
		region.SetSize(2, rangeZ->value());
		
		regView->setFixedWindowRegion(region);
		regView->setWindowingMode(true);
		regView->showWindowRegion(showRegionBox->isChecked());
		
		fixedMaxPerBoxChanged(regView->getFixedWindowMax()*100.0);
		fixedMinPerBoxChanged(regView->getFixedWindowMin()*100.0);
		movingMaxPerBoxChanged(regView->getMovingWindowMax()*100.0);
		movingMinPerBoxChanged(regView->getMovingWindowMin()*100.0);
	}

	regView->setFixedImageColorTable( (ColorTable) fixedColorCombo->currentIndex());
	regView->setMovingImageColorTable( (ColorTable) movingColorCombo->currentIndex());
	
	regView->updateView();
}

void RegistrationViewDialog::revert()
{
	InternalPixelType fixedMin = regView->getFixedImageMin();
	InternalPixelType fixedMax = regView->getFixedImageMax();
	InternalPixelType movingMin = regView->getMovingImageMin();
	InternalPixelType movingMax = regView->getMovingImageMax();
	
	logging::logMessage("Reverting to default intensity window values");
	
	manualWindowButton->setChecked(true);
	regView->setWindowingMode(false);
	regView->showWindowRegion(false);
	
	//See Issue #9 for explaination of the windowing formula
	fixedMaxPerBox->setValue(DEFAULT_WINDOW_MAX * 100.0);
	fixedMaxValBox->setValue(DEFAULT_WINDOW_MAX * (fixedMax - fixedMin) + fixedMin);
	fixedMinPerBox->setValue(DEFAULT_WINDOW_MIN * 100.0);
	fixedMinValBox->setValue(DEFAULT_WINDOW_MIN * (fixedMax - fixedMin) + fixedMin);
	movingMaxPerBox->setValue(DEFAULT_WINDOW_MAX * 100.0);
	movingMaxValBox->setValue(DEFAULT_WINDOW_MAX * (movingMax - movingMin) + movingMin);
	movingMinPerBox->setValue(DEFAULT_WINDOW_MIN * 100.0);
	movingMinValBox->setValue(DEFAULT_WINDOW_MIN * (movingMax - movingMin) + movingMin);
	
	RegionType region = regView->getFixedWindowRegion();
	indexX->setValue(region.GetIndex(0));
	indexY->setValue(region.GetIndex(1));
	indexZ->setValue(region.GetIndex(2));
	rangeX->setValue(region.GetSize(0));
	rangeY->setValue(region.GetSize(1));
	rangeZ->setValue(region.GetSize(2));
	showRegionBox->setChecked(false);
	
	regView->setFixedWindowMax(fixedMaxPerBox->value()/100.0);
	regView->setFixedWindowMin(fixedMinPerBox->value()/100.0);
	regView->setMovingWindowMax(movingMaxPerBox->value()/100.0);
	regView->setMovingWindowMin(movingMinPerBox->value()/100.0);
	fixedImageChanged();
	movingImageChanged();
	regView->updateView();
}

void RegistrationViewDialog::fixedMaxPerBoxChanged(double val)
{
	InternalPixelType fixedMax = regView->getFixedImageMax();
	InternalPixelType fixedMin = regView->getFixedImageMin();
	fixedMaxValBox->setValue( (fixedMax - fixedMin)* val / 100.0 + fixedMin);
}

void RegistrationViewDialog::fixedMaxValBoxChanged(double val)
{
	InternalPixelType fixedMax = regView->getFixedImageMax();
	InternalPixelType fixedMin = regView->getFixedImageMin();
	fixedMaxPerBox->setValue( (val - fixedMin) / (fixedMax - fixedMin) * 100.0);	
}

void RegistrationViewDialog::fixedMinPerBoxChanged(double val)
{
	InternalPixelType fixedMax = regView->getFixedImageMax();
	InternalPixelType fixedMin = regView->getFixedImageMin();
	fixedMinValBox->setValue( (fixedMax - fixedMin)* val / 100.0 + fixedMin);
}

void RegistrationViewDialog::fixedMinValBoxChanged(double val)
{
	InternalPixelType fixedMax = regView->getFixedImageMax();
	InternalPixelType fixedMin = regView->getFixedImageMin();
	fixedMinPerBox->setValue( (val - fixedMin) / (fixedMax - fixedMin) * 100.0);
}

void RegistrationViewDialog::movingMaxPerBoxChanged(double val)
{
	InternalPixelType movingMax = regView->getMovingImageMax();
	InternalPixelType movingMin = regView->getMovingImageMin();
	movingMaxValBox->setValue( (movingMax - movingMin)* val / 100.0 + movingMin);
}

void RegistrationViewDialog::movingMaxValBoxChanged(double val)
{
	InternalPixelType movingMax = regView->getMovingImageMax();
	InternalPixelType movingMin = regView->getMovingImageMin();
	movingMaxPerBox->setValue( (val - movingMin) / (movingMax - movingMin) * 100.0);			
}

void RegistrationViewDialog::movingMinPerBoxChanged(double val)
{
	InternalPixelType movingMax = regView->getMovingImageMax();
	InternalPixelType movingMin = regView->getMovingImageMin();
	movingMinValBox->setValue( (movingMax - movingMin)* val / 100.0 + movingMin);
}

void RegistrationViewDialog::movingMinValBoxChanged(double val)
{
	InternalPixelType movingMax = regView->getMovingImageMax();
	InternalPixelType movingMin = regView->getMovingImageMin();
	movingMinPerBox->setValue( (val - movingMin) / (movingMax - movingMin) * 100.0);
}

void RegistrationViewDialog::fixedPrecisionChanged(int val)
{
	fixedMaxValBox->setDecimals(val);
	fixedMinValBox->setDecimals(val);
}

void RegistrationViewDialog::movingPrecisionChanged(int val)
{
	movingMaxValBox->setDecimals(val);
	movingMinValBox->setDecimals(val);
}

void RegistrationViewDialog::fixedImageChanged()
{
	InternalPixelType fixedMin = regView->getFixedImageMin();
	InternalPixelType fixedMax = regView->getFixedImageMax();
	int decimals;
	
	//save the current min and max percent before we adjust the range and precision parameters
	float winMax = fixedMaxPerBox->value();
	float winMin = fixedMinPerBox->value();
	
	logging::logMessage("Fixed image changed in RegistraiontViewDialog");
	
	decimals = (int) ceil(log10(fixedMax)) - 6;
	fixedMaxValBox->setRange( - 10 * (fixedMax - fixedMin) + fixedMin, 10 * (fixedMax - fixedMin) + fixedMax);
	fixedMaxValBox->setSingleStep( (fixedMax - fixedMin) / 100.0);
	fixedMinValBox->setRange( - 10 * (fixedMax - fixedMin) + fixedMin, 10 * (fixedMax - fixedMin) + fixedMax);
	fixedMinValBox->setSingleStep( (fixedMax - fixedMin) / 100.0);
	fixedPrecisionBox->setValue( (decimals < 0) ? -decimals : 1);
		
	fixedMaxPerBoxChanged(winMax);
	fixedMinPerBoxChanged(winMin);
	
	InternalImageType::Pointer image = regView->getFixedImage();
	
	if(image.IsNull())
		return;
	
	RegionType region = image->GetLargestPossibleRegion();
	indexX->setRange(region.GetIndex(0), region.GetSize(0));
	indexY->setRange(region.GetIndex(1), region.GetSize(1));
	indexZ->setRange(region.GetIndex(2), region.GetSize(2));
	rangeX->setRange(region.GetIndex(0), region.GetSize(0));
	rangeY->setRange(region.GetIndex(1), region.GetSize(1));
	rangeZ->setRange(region.GetIndex(2), region.GetSize(2));
}

void RegistrationViewDialog::movingImageChanged()
{
	InternalPixelType movingMin = regView->getMovingImageMin();
	InternalPixelType movingMax = regView->getMovingImageMax();
	int decimals;
	
	//save the current min and max percent before we adjust the range and precision parameters
	float winMax = movingMaxPerBox->value();
	float winMin = movingMinPerBox->value();
	
	logging::logMessage("Moving image changed in RegistraiontViewDialog");
	
	decimals = (int) ceil(log10(movingMax)) - 6;
	movingMaxValBox->setRange( -10 * (movingMax - movingMin) + movingMin, 10 * (movingMax - movingMin) + movingMax);
	movingMaxValBox->setSingleStep( (movingMax - movingMin) / 100.0);
	movingMinValBox->setRange( -10 * (movingMax - movingMin) + movingMin, 10 * (movingMax - movingMin) + movingMax);
	movingMinValBox->setSingleStep( (movingMax - movingMin) / 100.0);
	movingPrecisionBox->setValue( (decimals < 0) ? -decimals : 1);
	
	movingMaxPerBoxChanged(winMax);
	movingMinPerBoxChanged(winMin);
}

void RegistrationViewDialog::manualWindowMode()
{
	indexX->setEnabled(false);
	indexY->setEnabled(false);
	indexZ->setEnabled(false);
	rangeX->setEnabled(false);
	rangeY->setEnabled(false);
	rangeZ->setEnabled(false);
	
	fixedMaxPerBox->setEnabled(true);
	fixedMaxValBox->setEnabled(true);
	fixedMinPerBox->setEnabled(true);
	fixedMinValBox->setEnabled(true);
	fixedPrecisionBox->setEnabled(true);
	
	movingMaxPerBox->setEnabled(true);
	movingMaxValBox->setEnabled(true);
	movingMinPerBox->setEnabled(true);
	movingMinValBox->setEnabled(true);
	movingPrecisionBox->setEnabled(true);
}

void RegistrationViewDialog::regionWindowMode()
{
	indexX->setEnabled(true);
	indexY->setEnabled(true);
	indexZ->setEnabled(true);
	rangeX->setEnabled(true);
	rangeY->setEnabled(true);
	rangeZ->setEnabled(true);
	
	fixedMaxPerBox->setEnabled(false);
	fixedMaxValBox->setEnabled(false);
	fixedMinPerBox->setEnabled(false);
	fixedMinValBox->setEnabled(false);
	fixedPrecisionBox->setEnabled(false);
	
	movingMaxPerBox->setEnabled(false);
	movingMaxValBox->setEnabled(false);
	movingMinPerBox->setEnabled(false);
	movingMinValBox->setEnabled(false);
	movingPrecisionBox->setEnabled(false);
}


