//
//  $Id$  
//  $Lastlog$
//
//  GuiIntensityWindowTest.cpp
//  petspectfusionTest
//
//	Created 02/11/2009, This is a test case that verifies that the intensity windowing works correctly.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <fstream>
#include <iostream>
#include <sstream>

#include <QtGui>
#include <QTest>
#include <QSignalSpy>
#include <cppunit/extensions/HelperMacros.h>

#include "ImageIO.h"
#include "GuiIntensityWindowTest.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "RegistrationView.h"
#include "RegistrationViewDialog.h"

using namespace std;
using namespace PetSpectFusionApp;

void GuiIntensityWindowTest::testIntensityWindowCore()
{
	QVERIFY(sourceLocation != NULL);
	QVERIFY(fixedFileName.size() > 0);
	QVERIFY(movingFileName.size() > 0);
	
	InternalImageType::Pointer fixedImage = loadMhdImage(fixedFileName.c_str());	
	QVERIFY( fixedImage.IsNotNull() );
	
	InternalImageType::Pointer movingImage = loadMhdImage(movingFileName.c_str());
	QVERIFY( movingImage.IsNotNull() );

	//load the images and verify the max and min values are correct
	typedef itk::MinimumMaximumImageCalculator<InternalImageType> MinMaxCalcType;
	MinMaxCalcType::Pointer minMaxCalc = MinMaxCalcType::New();
	minMaxCalc->SetImage(fixedImage);
	minMaxCalc->SetRegion(fixedImage->GetLargestPossibleRegion());
	minMaxCalc->Compute();
	
	InternalPixelType fixedImageMin = minMaxCalc->GetMinimum();
	InternalPixelType fixedImageMax = minMaxCalc->GetMaximum();
	QCOMPARE( fixedImageMax, (float) 72087.0);
	QCOMPARE( fixedImageMin, (float) -13415.6);
	
	minMaxCalc->SetImage(movingImage);
	minMaxCalc->SetRegion(movingImage->GetLargestPossibleRegion());
	minMaxCalc->Compute();
	
	InternalPixelType movingImageMin = minMaxCalc->GetMinimum();
	InternalPixelType movingImageMax = minMaxCalc->GetMaximum();
	QCOMPARE( movingImageMax, (float) 471.0);
	QCOMPARE( movingImageMin, (float) 0.0);
	
	//load the images into the reg view and verify the max and min values are correct
	QRegistrationView* regView = new QRegistrationView;
	regView->setFixedImage(fixedImage);
	regView->setMovingImage(movingImage);
	
	QCOMPARE( regView->getFixedImageMax(), fixedImageMax);
	QCOMPARE( regView->getFixedImageMin(), fixedImageMin);
	QCOMPARE( regView->getMovingImageMax(), movingImageMax);
	QCOMPARE( regView->getMovingImageMin(), movingImageMin);
	
	//create the view options dialog, and verify the values are set corectly
	RegistrationViewDialog* viewDialog = new RegistrationViewDialog(NULL, regView);
	viewDialog->fixedMaxValBox->setValue(63536.6);
	QCOMPARE( 90.0, viewDialog->fixedMaxPerBox->value());

	viewDialog->fixedMinValBox->setValue(-4865.34);
	QCOMPARE( 10.0, viewDialog->fixedMinPerBox->value());
	
	QTest::mouseClick(viewDialog->applyButton, Qt::LeftButton);
	QCOMPARE(regView->getFixedWindowMax(), (float) 0.9);
	QCOMPARE(regView->getFixedWindowMin(), (float) 0.1);
	
	viewDialog->movingMaxValBox->setValue(353.25);
	QCOMPARE( viewDialog->movingMaxPerBox->value(), 75.0);
	
	viewDialog->movingMinValBox->setValue(117.75);
	QCOMPARE( viewDialog->movingMinPerBox->value(), 25.0);
	
	QTest::mouseClick(viewDialog->applyButton, Qt::LeftButton);
	QCOMPARE(regView->getMovingWindowMax(), (float) 0.75);
	QCOMPARE(regView->getMovingWindowMin(), (float) 0.25);
	
	//test revert
	QTest::mouseClick(viewDialog->revertButton, Qt::LeftButton);
	QCOMPARE( (float) viewDialog->fixedMaxValBox->value(), (float) DEFAULT_WINDOW_MAX * (fixedImageMax - fixedImageMin) + fixedImageMin);
	QCOMPARE( (float) viewDialog->fixedMinValBox->value(), (float) DEFAULT_WINDOW_MIN * (fixedImageMax - fixedImageMin) + fixedImageMin);
	QCOMPARE( (float) viewDialog->movingMaxValBox->value(), (float) DEFAULT_WINDOW_MAX * (movingImageMax - movingImageMin) + movingImageMin);
	QCOMPARE( (float) viewDialog->movingMaxValBox->value(), (float) DEFAULT_WINDOW_MAX * (movingImageMax - movingImageMin) + movingImageMin);

	QCOMPARE( regView->getFixedWindowMax(), (float) DEFAULT_WINDOW_MAX);
	QCOMPARE( regView->getFixedWindowMin(), (float) DEFAULT_WINDOW_MIN);
	QCOMPARE( regView->getMovingWindowMax(), (float) DEFAULT_WINDOW_MAX);
	QCOMPARE( regView->getMovingWindowMin(), (float) DEFAULT_WINDOW_MIN);
	
	delete regView;

}
