//
//  $Id$  
//  $Lastlog$
//
//  GuiRegistrationResultsTest.cpp
//  petspectfusionTest
//
//	Created 29/10/2009, This test case verifies that all the registration modes deliver the same results
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <fstream>
#include <iostream>
#include <sstream>
#include <string.h>

#include <QtConcurrentRun>
#include <QtGui>
#include <QSignalSpy>

#include "ImageIO.h"
#include "GuiRegistrationResultsTest.h"
#include <cppunit/extensions/HelperMacros.h>
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "Registration.h"
#include "RegObserver.h"
#include "typedefs.h"

using namespace std;
using namespace PetSpectFusionApp;

void GuiRegistrationResultsTest::testRegistrationResults()
{		
	QVERIFY(fixedFileName.size() != 0);
	QVERIFY(movingFileName.size() != 0);
	QVERIFY(outputFileName.size() != 0);
		
	PetSpectFusionApp::automatic_mode = true;
	
	//load the images set the result of the necessary params
	InternalImageType::Pointer fixedImage = loadMhdImage(fixedFileName.c_str());
	InternalImageType::Pointer movingImage = loadMhdImage(movingFileName.c_str());
	
	// Create the Command observer for watching the registration
	CommandIterationUpdate::Pointer observer = CommandIterationUpdate::New();
	
	//first perform fully automatic headless mode
	RegistrationResultType resultHeadless =  doMattesRegistration(fixedImage, movingImage, observer, bins,
												sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales);

	QCOMPARE(resultHeadless.error, 0);
	
	QPetSpectFusionViewer* autoViewer = new QPetSpectFusionViewer(fixedFileName, movingFileName, outputFileName, bins, 
																  sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);

	QCOMPARE(autoViewer->getFixedFileName(), fixedFileName);
	QCOMPARE(autoViewer->getMovingFileName(), movingFileName);
	QCOMPARE(autoViewer->getOutputFileName(), outputFileName);
	
	//now perform fully auto GUI based registration
	autoViewer->doAutomaticReg();
	
	QTest::qWait(1000);
	
	for(int i = 0; i < 60; i++)
		if(autoViewer->isRegistrationRunning() == false)
			break;
		else 
			QTest::qWait(1000);
	
	QVERIFY(autoViewer->isRegistrationRunning() == false);
		
	//check that the registration results match
	QCOMPARE(resultHeadless.iteration, autoViewer->getIteration());
	QCOMPARE((float) resultHeadless.metricVal, (float) autoViewer->getMetricVal());
	QCOMPARE(resultHeadless.params, autoViewer->getParameters());
	
	QPetSpectFusionViewer* manualViewer = new QPetSpectFusionViewer(string(""), string(""), outputFileName, bins, 
																   sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);
	
	manualViewer->openFixedFileName(fixedFileName);
	manualViewer->openMovingFileName(movingFileName);
	manualViewer->doReg();
	
	QTest::qWait(1000);
	
	for(int i = 0; i < 60; i++)
		if(manualViewer->isRegistrationRunning() == false)
			break;
		else 
			QTest::qWait(1000);

	QVERIFY(manualViewer->isRegistrationRunning() == false);

	
	QCOMPARE(resultHeadless.iteration, manualViewer->getIteration());
	QCOMPARE((float) resultHeadless.metricVal, (float) manualViewer->getMetricVal());
	QCOMPARE(resultHeadless.params, manualViewer->getParameters());
	
	delete autoViewer;
	delete manualViewer;
}


