//
//  $Id: GuiVOIRegistrationTest.cpp 58:d0805496a90b 2009-12-12 20:42 +0100 jensen $  
//  $Lastlog: VOI Unit test expanded refs #25 $
//
//  GuiVOIRegistrationTest.cpp
//  petspectfusionTest
//
//	Created 10/12/2009, This is a test case that checks the VOI registration mode of the GUI.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <QtConcurrentRun>
#include <QtGui>
#include <QTest>
#include <cppunit/extensions/HelperMacros.h>

#include "GuiVOIRegistrationTest.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "RegSettingsDialog.h"

using namespace std;
using namespace PetSpectFusionApp;

void GuiVOIRegistrationTest::testGuiVOIRegisration()
{
	QVERIFY(sourceLocation != NULL);
	QVERIFY(fixedFileName.size() > 0);
	QVERIFY(movingFileName.size() > 0);

	//create a viewer object and open the settings dialog
	mainViewer = new QPetSpectFusionViewer(fixedFileName, movingFileName, outputFileName, bins, 
															  sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);
	
	mainViewer->showSettingsDialog();
	RegSettingsDialog* settingsDialog = mainViewer->getRegSettingsDialog();
	settingsDialog->hide();
	
	settingsDialog->indexX->setValue(65);
	settingsDialog->rangeX->setValue(50);
	settingsDialog->indexY->setValue(30);
	settingsDialog->rangeY->setValue(45);
	settingsDialog->indexZ->setValue(15);
	settingsDialog->rangeZ->setValue(40);
	settingsDialog->applyButton->click();
	
	//verify the region was set correctly
	RegionType region = mainViewer->getVOIRegion();
	QCOMPARE( (int) region.GetIndex(0), 65);
	QCOMPARE( (int) region.GetIndex(1), 30);
	QCOMPARE( (int) region.GetIndex(2), 15);
	QCOMPARE( (int) region.GetSize(0), 50);
	QCOMPARE( (int) region.GetSize(1), 45);
	QCOMPARE( (int) region.GetSize(2), 40);
	
	//test that the VOI registration mode works
	mainViewer->setVoiMode();
	mainViewer->doReg();
	
	QVERIFY(mainViewer->isRegistrationRunning() == true);
	
	for(int i = 0; i < 60; i++)
		if(mainViewer->isRegistrationRunning() == false)
			break;
		else 
			QTest::qWait(1000);
	
	QVERIFY(mainViewer->isRegistrationRunning() == false);
	

	//check that invalid input is handled properly
	settingsDialog->indexX->setValue(0);
	settingsDialog->rangeX->setValue(0);
	settingsDialog->indexY->setValue(0);
	settingsDialog->rangeY->setValue(0);
	settingsDialog->indexZ->setValue(0);
	settingsDialog->rangeZ->setValue(0);
	settingsDialog->applyButton->click();
	
	//mainViewer->doReg blocks if the region size is invalid
	QtConcurrent::run(this, &GuiVOIRegistrationTest::testInvalidVOIBoundsResponse);
	
	mainViewer->doReg();

	delete mainViewer;

}

void GuiVOIRegistrationTest::testInvalidVOIBoundsResponse()
{
	
	QTest::qWait(100);
	
	//find the popup message box
	QWidget* messageWindow = QApplication::activeModalWidget();
	if (!messageWindow)
		foreach (QWidget *widget, QApplication::topLevelWidgets())
		if (widget->isModal())
            messageWindow=widget;
	if (!messageWindow)
		return;	//popup not found, should cause test error
	
	QTest::keyClick(messageWindow, Qt::Key_Escape); 

}

void GuiVOIRegistrationTest::testGuiAutomaticVOIRegistration()
{
	
	QVERIFY(sourceLocation != NULL);
	QVERIFY(fixedFileName.size() > 0);
	QVERIFY(movingFileName.size() > 0);
	
	char* argv[8];  	
	argv[0] = (char *) malloc(25);
	strncpy(argv[0], "petspectfusionTest", 25); 
	argv[1] = (char *) malloc(25);
	strncpy(argv[1], "-voifull", 25); 
	argv[2] = (char *) malloc(25);
	strncpy(argv[2], "65", 25); 
	argv[3] = (char *) malloc(25);
	strncpy(argv[3], "30", 25);
	argv[4] = (char *) malloc(25);
	strncpy(argv[4], "15", 25);
	argv[5] = (char *) malloc(25);
	strncpy(argv[5], "50", 25); 
	argv[6] = (char *) malloc(25);
	strncpy(argv[6], "45", 25); 
	argv[7] = (char *) malloc(25);
	strncpy(argv[7], "40", 25); 
	
	PetSpectFusionApp::processCommandLine(7, argv);
	QVERIFY( voiRegion.GetIndex(0) == 65);
	QVERIFY( voiRegion.GetIndex(1) == 30);
	QVERIFY( voiRegion.GetIndex(2) == 15);
	QVERIFY( voiRegion.GetSize(0) == 50);
	QVERIFY( voiRegion.GetSize(1) == 45);
	QVERIFY( voiRegion.GetSize(2) == 40);
	QVERIFY( doVOIAndFullRegistration == true);
	
	//create a viewer object
	mainViewer = new QPetSpectFusionViewer(fixedFileName, movingFileName, outputFileName, bins, 
										   sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);
	
	//trigger the automatic registration
	mainViewer->doAutomaticReg();
	
	QTest::qWait(300);
	
	QVERIFY(mainViewer->isRegistrationRunning() == true);
	
	for(int i = 0; i < 60; i++)
		if(mainViewer->isRegistrationRunning() == false)
			break;
		else 
			QTest::qWait(1000);
	
	QVERIFY(mainViewer->isRegistrationRunning() == false);
	QVERIFY(mainViewer->getIteration() > 0);
	
	delete mainViewer;
}




