//
//  $Id$  
//  $Lastlog$
//
//  GuiWindowRegionTest.cpp
//  petspectfusionTest
//
//	Created 02/11/2009, This is a test case that verifies that the intensity windowing works correctly.
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <fstream>
#include <iostream>
#include <sstream>

#include <QtGui>
#include <QTest>
#include <cppunit/extensions/HelperMacros.h>

#include "GuiWindowRegionTest.h"
#include "PetSpectFusionApp.h"
#include "PetSpectFusionViewer.h"
#include "RegistrationView.h"
#include "RegistrationViewDialog.h"

using namespace std;
using namespace PetSpectFusionApp;

void GuiWindowRegionTest::testGuiWindowRegion()
{
	QVERIFY(sourceLocation != NULL);
	QVERIFY(fixedFileName.size() > 0);
	QVERIFY(movingFileName.size() > 0);
		
	QPetSpectFusionViewer* viewer = new QPetSpectFusionViewer(fixedFileName, movingFileName, outputFileName, bins, 
															  sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);
	
	QVERIFY(viewer != NULL);
	QRegistrationView* regView = viewer->getRegistrationView();
	QVERIFY(viewer != NULL);
	
	//create the view options dialog
	RegistrationViewDialog* viewDialog = new RegistrationViewDialog(viewer, regView);
	QVERIFY(viewDialog != NULL);
	
	//switch windowing mode and verify that the window region results are correct
	viewDialog->regionWindowButton->setChecked(true);
	viewDialog->indexX->setValue(75);
	viewDialog->indexY->setValue(35);
	viewDialog->indexZ->setValue(20);
	viewDialog->rangeX->setValue(35);
	viewDialog->rangeY->setValue(35);
	viewDialog->rangeZ->setValue(30);
	viewDialog->applyButton->click();
	
	//check the window values in the reg view
	QCOMPARE(regView->getFixedWindowMax(), (float) 1);
	QCOMPARE(regView->getFixedWindowMin(), (float) 0.138917);
	QCOMPARE(regView->getMovingWindowMax(), (float) 0.767253);
	QCOMPARE(regView->getMovingWindowMin(), (float) 0.0);
	
	//verify that the dialog values match
	QCOMPARE(viewDialog->fixedMaxPerBox->value(), 100.0);
	QCOMPARE(viewDialog->fixedMinPerBox->value(), 13.89);
	QCOMPARE(viewDialog->movingMaxPerBox->value(), 76.73);
	QCOMPARE(viewDialog->movingMinPerBox->value(), 0.0);
	
	//move the moving image and verify that the values change accordingly
	ParametersType params(6);
	params.SetElement(0, 0);
	params.SetElement(1, 0);
	params.SetElement(2, 0);
	params.SetElement(3, 131);
	params.SetElement(4, 71);
	params.SetElement(5, 20);
	regView->setTransformParams(params);
	
	//trigger an update of the dialog
	viewDialog->applyButton->click();
	
	//sanity check: fixed image window should not have changed
	QCOMPARE(viewDialog->fixedMaxPerBox->value(), 100.0);
	QCOMPARE(viewDialog->fixedMinPerBox->value(), 13.89);
	
	//check the new moving window
	QCOMPARE(viewDialog->movingMaxPerBox->value(), 59.53);
	QCOMPARE(viewDialog->movingMinPerBox->value(), 0.0);
	
	delete viewer;
}

void GuiWindowRegionTest::testCommandLineWindowRegion()
{
	QVERIFY(sourceLocation != NULL);
	QVERIFY(fixedFileName.size() > 0);
	QVERIFY(movingFileName.size() > 0);
	
	//set the fixed window region, as if it were set per command line
	fixedWindowRegion.SetIndex(0, 75);
	fixedWindowRegion.SetIndex(1, 35);
	fixedWindowRegion.SetIndex(2, 20);
	fixedWindowRegion.SetSize(0, 35);
	fixedWindowRegion.SetSize(1, 35);
	fixedWindowRegion.SetSize(2, 30);
		
	QPetSpectFusionViewer* viewer = new QPetSpectFusionViewer(fixedFileName, movingFileName, outputFileName, bins, 
																  sampleRate, minStepSize, maxStepSize, maxIterations, optimizerScales, zoomFactor);

	QVERIFY(viewer != NULL);
	QRegistrationView* regView = viewer->getRegistrationView();
	QVERIFY(regView != NULL);
	
	QVERIFY(regView->getWindowingMode() == true);
	QCOMPARE(regView->getFixedWindowMax(), (float) 1);
	QCOMPARE(regView->getFixedWindowMin(), (float) 0.138917);
	QCOMPARE(regView->getMovingWindowMax(), (float) 0.767253);
	QCOMPARE(regView->getMovingWindowMin(), (float) 0.0);
	
	//make sure moving window changes when the image moves
	ParametersType params(6);
	params.SetElement(0, 0);
	params.SetElement(1, 0);
	params.SetElement(2, 0);
	params.SetElement(3, 131);
	params.SetElement(4, 71);
	params.SetElement(5, 20);
	regView->setTransformParams(params);
	
	//sanity check, fixed window should not have changed
	QCOMPARE(regView->getFixedWindowMax(), (float) 1);
	QCOMPARE(regView->getFixedWindowMin(), (float) 0.138917);
	QCOMPARE(regView->getMovingWindowMax(), (float) 0.595265);
	QCOMPARE(regView->getMovingWindowMin(), (float) 0);
	
	delete viewer;
}

