//
//  $Id$
//  $Lastlog$
//
//  LogTest.cpp
//  petspectfusionTest
//
//	Created 27/10/2009, unit test cases for checking
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include <fstream>
#include <iostream>
#include <sstream>

#include "LogTest.h"
#include "Log.h"
#include "PetSpectFusionViewer.h"

using namespace std;
using namespace logging;

static ostringstream ostring(ostringstream::out);
static std::streambuf *outbuf, *errbuf;

// Add the core test suite to the test registry
CPPUNIT_TEST_SUITE_REGISTRATION( LogTest );

//swap
void LogTest::setUp()
{
	outbuf = cout.rdbuf();
	errbuf = cerr.rdbuf();
	cout.rdbuf(ostring.rdbuf());
	cerr.rdbuf(ostring.rdbuf());
}

void LogTest::tearDown()
{
	cout.rdbuf(outbuf);
	cerr.rdbuf(errbuf);
}

void LogTest::testLogMessage()
{
	logMessage("Test Message");
	string logString = ostring.str();
	int pos = logString.find("Test Message");
	CPPUNIT_ASSERT(pos > 0);
	CPPUNIT_ASSERT_EQUAL(pos, 23);
	ostring.str("");
}

void LogTest::testLogError()
{
	logError("Test Message");
	string logString = ostring.str();
	int pos = logString.find("Test Message");
	CPPUNIT_ASSERT(pos > 0);
	CPPUNIT_ASSERT_EQUAL(pos, 23);
	ostring.str("");
}

void LogTest::testLogFile()
{
	//first turn on file logging
	string fileName("output.log");
	enableFileLogging(fileName);
	
	//verify that log can opened and written to
	CPPUNIT_ASSERT(fileLoggingEnabled());
	
	logMessage("Test Message");
	logError("Error Message");
	disableFileLogging();
	
	CPPUNIT_ASSERT(!fileLoggingEnabled());
	
	//now check the results
	ifstream logFile;
	logFile.open(fileName.c_str());
	
	char line[256], *pos;
	
	logFile.getline(line, 256);
	pos = strstr(line, "Logging started");
	cout << pos;
	CPPUNIT_ASSERT(pos != NULL);
	
	logFile.getline(line, 256);
	pos = strstr(line, "Test Message");
	cout << pos;
	CPPUNIT_ASSERT(pos != NULL);
	
	logFile.getline(line, 256);
	pos = strstr(line, "Error Message");
	cout << pos;
	CPPUNIT_ASSERT(pos != NULL);
	
	//verify that log messages are redirected back to standard output
	logFile.close();
	logMessage("Test Message");
	CPPUNIT_ASSERT(ostring.str().find("Test Message") > 0);
}